@@ -3,7 +3,7 @@ package Stream::Reader;
 use 5.005;
 use strict;
 
-our $VERSION = '0.06';
+our $VERSION = '0.09';
 
 # Global/system variables
 
@@ -18,19 +18,19 @@ $CODE ||= {
 new => <<'ENDC',
   my $class = shift;
   my $input = shift;
-  my $param = ( ref($Shift = shift) eq 'HASH' )? $Shift : {};
+  my $attr  = ( ref($Shift = shift) eq 'HASH' )? $Shift : {};
   my $self  = {
       # System parameters
     input    => $input,
     inpos    => 0,
-    inlimit  => ( defined($param->{Limit}) and $param->{Limit} >= 0 )? $param->{Limit} : 1e10,
-    buffsize => ( defined($param->{BuffSize}) and $param->{BuffSize} >= 0 )? $param->{BuffSize} : 32_768,
+    inlimit  => ( defined($attr->{Limit}) and $attr->{Limit} >= 0 )? $attr->{Limit} : 1e10,
+    buffsize => ( defined($attr->{BuffSize}) and $attr->{BuffSize} >= 0 )? $attr->{BuffSize} : 32_768,
     bufferA  => '',
     bufferB  => '',
     status   => 1,
       # System flags
-    mode_B  => ( $param->{Mode} and index(uc($param->{Mode}),'B') != -1 ),
-    mode_U  => ( $param->{Mode} and index(uc($param->{Mode}),'U') != -1 ),
+    mode_B  => ( $attr->{Mode} and index(uc($attr->{Mode}),'B') != -1 ),
+    mode_U  => ( $attr->{Mode} and index(uc($attr->{Mode}),'U') != -1 ),
       # Statistic parameters
     Match  => '',
     Readed => 0,
@@ -50,9 +50,9 @@ ENDC
 readto => <<'ENDC',
   my $self  = shift;
   my $delim = ( ref($Shift = shift) eq 'ARRAY' )? $Shift : [$Shift];
-  my $param = ( ref($Shift = shift) eq 'HASH' )? $Shift : {};
-  my $limit = ( defined($param->{Limit}) and $param->{Limit} >= 0 )? $param->{Limit} : 1e10;
-  my $wcase = ( $param->{Mode} and index(uc($param->{Mode}),'I') != -1 );
+  my $attr  = ( ref($Shift = shift) eq 'HASH' )? $Shift : {};
+  my $limit = ( defined($attr->{Limit}) and $attr->{Limit} >= 0 )? $attr->{Limit} : 1e10;
+  my $wcase = ( $attr->{Mode} and index(uc($attr->{Mode}),'I') != -1 );
   my $max_d = 0;
   my $min_d = 1e10;
   my $error;
@@ -62,10 +62,10 @@ readto => <<'ENDC',
   #  - reseting some statistic variables
   @$self{ qw(Readed Stored Match) } = ( (0)x2, '' );
   #  - initialize output stream, if this is SCALAR and initialization required
-  if( UNIVERSAL::isa($param->{Out},'SCALAR')
-    and !( defined(${$param->{Out}}) and $param->{Mode} and index(uc($param->{Mode}),'A') != -1 )
+  if( UNIVERSAL::isa($attr->{Out},'SCALAR')
+    and !( defined(${$attr->{Out}}) and $attr->{Mode} and index(uc($attr->{Mode}),'A') != -1 )
   ) {
-    ${$param->{Out}} = '';
+    ${$attr->{Out}} = '';
   }
   #  - maximal and minimal delimiter length detection
   foreach( @$delim ) {
@@ -73,71 +73,74 @@ readto => <<'ENDC',
     $min_d = length if $min_d > length;
   }
   #  - checking status and delimiter(s) presents
-  return $self->{status} unless( $self->{status} and $max_d );
-
-  # Processing:
-  while(1) {
-    #  - searching
-    if( length($self->{bufferA}) >= $min_d ) {
-      my $found = 1e10;
-      my $buffer;
-      $buffer = \( $self->{mode_B}? $self->{bufferB} : lc($self->{bufferA}) ) if $wcase;
-      foreach( @$delim ) {
-        my $pos = $wcase? index($$buffer,lc) : index($self->{bufferA},$_);
-        if( $pos != -1 and $pos < $found ) {
-          $found = $pos;
-          $self->{Match} = $_;
+  unless( $self->{status} and $max_d ) {
+    return $self->{status};
+  } else {
+    # Processing:
+    while(1) {
+      #  - searching
+      if( length($self->{bufferA}) >= $min_d ) {
+        my $found = 1e10;
+        my $buffer;
+        if( $wcase ) {
+          $buffer = \( $self->{mode_B}? $self->{bufferB} : lc($self->{bufferA}) );
+        }
+        foreach( @$delim ) {
+          my $pos = $wcase? index($$buffer,lc) : index($self->{bufferA},$_);
+          if( $pos != -1 and $pos < $found ) {
+            $found = $pos;
+            $self->{Match} = $_;
+          }
+        }
+        if( $found < 1e10 ) {
+          if( !$error and $self->{Stored} < $limit ) {
+            $rsize = $found;
+            $rsize = $limit - $self->{Stored} if( $rsize > $limit - $self->{Stored} );
+            $error = !$self->_write( $attr->{Out}, \(substr( $self->{bufferA}, 0, $rsize )) );
+            $self->{Stored} += $rsize unless $error;
+          }
+          $self->{Readed} += $found;
+          $self->{Total}  += $found;
+          my $psize = $found + length($self->{Match});
+          substr( $self->{bufferA}, 0, $psize, '' );
+          substr( $self->{bufferB}, 0, $psize, '' ) if $self->{mode_B};
+          return 1;
         }
       }
-      if( $found < 1e10 ) {
+      #  - move part data to output stream
+      if( length($self->{bufferA}) >= $max_d ) {
+        my $psize = length($self->{bufferA}) - ($max_d - 1);
         if( !$error and $self->{Stored} < $limit ) {
-          $rsize = $found;
+          $rsize = $psize;
           $rsize = $limit - $self->{Stored} if( $rsize > $limit - $self->{Stored} );
-          $error = !$self->_write( $param->{Out}, \(substr( $self->{bufferA}, 0, $rsize )) );
+          $error = !$self->_write( $attr->{Out}, \(substr( $self->{bufferA}, 0, $rsize )) );
           $self->{Stored} += $rsize unless $error;
         }
-        $self->{Readed} += $found;
-        $self->{Total}  += $found;
-        my $psize = $found + length($self->{Match});
+        $self->{Readed} += $psize;
+        $self->{Total}  += $psize;
         substr( $self->{bufferA}, 0, $psize, '' );
         substr( $self->{bufferB}, 0, $psize, '' ) if $self->{mode_B};
-        return 1;
       }
-    }
-    #  - move part data to output stream
-    if( length($self->{bufferA}) >= $max_d ) {
-      my $psize = length($self->{bufferA}) - ($max_d - 1);
-      if( !$error and $self->{Stored} < $limit ) {
-        $rsize = $psize;
-        $rsize = $limit - $self->{Stored} if( $rsize > $limit - $self->{Stored} );
-        $error = !$self->_write( $param->{Out}, \(substr( $self->{bufferA}, 0, $rsize )) );
-        $self->{Stored} += $rsize unless $error;
-      }
-      $self->{Readed} += $psize;
-      $self->{Total}  += $psize;
-      substr( $self->{bufferA}, 0, $psize, '' );
-      substr( $self->{bufferB}, 0, $psize, '' ) if $self->{mode_B};
-    }
-    #  - if limit not ended yet then trying to fill buffer
-    #  - else move last data to output stream and finish
-    if( $self->{inlimit} ) {
-      return 0 unless $self->_fill_buffer();
-    }
-    else {
-      if( length $self->{bufferA} ) {
-        $rsize = length $self->{bufferA};
-        $self->{Readed} += $rsize;
-        $self->{Total}  += $rsize;
-        if( !$error and $self->{Stored} < $limit ) {
-          $rsize = $limit - $self->{Stored} if( $rsize > $limit - $self->{Stored} );
-          $error = !$self->_write( $param->{Out}, \(substr( $self->{bufferA}, 0, $rsize )) );
-          $self->{Stored} += $rsize unless $error;
+      #  - if limit not ended yet then trying to fill buffer
+      #  - else move last data to output stream and finish
+      if( $self->{inlimit} ) {
+        return 0 unless $self->_fill_buffer();
+      } else {
+        if( length $self->{bufferA} ) {
+          $rsize = length $self->{bufferA};
+          $self->{Readed} += $rsize;
+          $self->{Total}  += $rsize;
+          if( !$error and $self->{Stored} < $limit ) {
+            $rsize = $limit - $self->{Stored} if( $rsize > $limit - $self->{Stored} );
+            $error = !$self->_write( $attr->{Out}, \(substr( $self->{bufferA}, 0, $rsize )) );
+            $self->{Stored} += $rsize unless $error;
+          }
+          $self->{bufferA} = '';
+          $self->{bufferB} = '' if $self->{mode_B};
         }
-        $self->{bufferA} = '';
-        $self->{bufferB} = '' if $self->{mode_B};
+        $self->{status} = 0;
+        return( ( $attr->{Mode} and index(uc($attr->{Mode}),'E') != -1 )? 0 : 1 );
       }
-      $self->{status} = 0;
-      return( ( $param->{Mode} and index(uc($param->{Mode}),'E') != -1 )? 0 : 1 );
     }
   }
 ENDC
@@ -146,7 +149,7 @@ ENDC
 readsome => <<'ENDC',
   my $self  = shift;
   my $limit = ( defined($Shift = shift) and $Shift >= 0 )? $Shift : 1e10;
-  my $param = ( ref($Shift = shift) eq 'HASH' )? $Shift : {};
+  my $attr  = ( ref($Shift = shift) eq 'HASH' )? $Shift : {};
   my $rsize;
   my $error;
 
@@ -154,45 +157,49 @@ readsome => <<'ENDC',
   #  - reseting some statistic variables
   @$self{ qw(Readed Stored Match) } = ( (0)x2, '' );
   #  - initialize output stream, if this is SCALAR and initialization required
-  if( UNIVERSAL::isa($param->{Out},'SCALAR')
-    and !( defined(${$param->{Out}}) and $param->{Mode} and index(uc($param->{Mode}),'A') != -1 )
+  if( UNIVERSAL::isa($attr->{Out},'SCALAR')
+    and !( defined(${$attr->{Out}}) and $attr->{Mode} and index(uc($attr->{Mode}),'A') != -1 )
   ) {
-    ${$param->{Out}} = '';
+    ${$attr->{Out}} = '';
   }
   #  - checking status
-  return 0 unless $self->{status};
-
-  # Processing:
-  while( $self->{Readed} < $limit ) {
-    #  - trying to fill buffer
-    unless( length $self->{bufferA} ) {
-      return 0 unless $self->_fill_buffer();
-    }
-    #  - if buffer still empty then break cycle
-    #  - else if not enouth data in buffer, then move all data from buffer to output stream
-    #  - else move necessary of characters to output stream  and break cycle
-    unless( length $self->{bufferA} ) {
-      $self->{status} = 0;
-      return( $self->{Readed} ? 1 : 0 );
-    }
-    elsif( length($self->{bufferA}) <= $limit - $self->{Readed} ) {
-      $error = !$self->_write( $param->{Out}, \($self->{bufferA}) ) unless $error;
-      $rsize = length $self->{bufferA};
-      $self->{Stored} += $rsize unless $error;
-      $self->{Readed} += $rsize;
-      $self->{Total}  += $rsize;
-      $self->{bufferA} = '';
-      $self->{bufferB} = '' if $self->{mode_B};
-    }
-    else {
-      $rsize = $limit - $self->{Readed};
-      $error = !$self->_write( $param->{Out}, \(substr( $self->{bufferA}, 0, $rsize )) ) unless $error;
-      $self->{Stored} += $rsize unless $error;
-      $self->{Readed} += $rsize;
-      $self->{Total}  += $rsize;
-      substr( $self->{bufferA}, 0, $rsize, '' );
-      substr( $self->{bufferB}, 0, $rsize, '' ) if $self->{mode_B};
-      last;
+  unless( $self->{status} ) {
+    return 0;
+  } else {
+    # Processing:
+    while( $self->{Readed} < $limit ) {
+      #  - trying to fill buffer
+      unless( length $self->{bufferA} ) {
+        return 0 unless $self->_fill_buffer();
+      }
+      #  - if buffer still empty then break cycle
+      unless( length $self->{bufferA} ) {
+        $self->{status} = 0;
+        return( $self->{Readed} ? 1 : 0 );
+      #  - else if not enouth data in buffer, then move all data from buffer to output stream
+      } elsif( length($self->{bufferA}) <= $limit - $self->{Readed} ) {
+        unless( $error ) {
+          $error = !$self->_write( $attr->{Out}, \($self->{bufferA}) );
+        }
+        $rsize = length $self->{bufferA};
+        $self->{Stored} += $rsize unless $error;
+        $self->{Readed} += $rsize;
+        $self->{Total}  += $rsize;
+        $self->{bufferA} = '';
+        $self->{bufferB} = '' if $self->{mode_B};
+      #  - else move necessary of characters to output stream  and break cycle
+      } else {
+        $rsize = $limit - $self->{Readed};
+        unless( $error ) {
+          $error = !$self->_write( $attr->{Out}, \(substr( $self->{bufferA}, 0, $rsize )) );
+        }
+        $self->{Stored} += $rsize unless $error;
+        $self->{Readed} += $rsize;
+        $self->{Total}  += $rsize;
+        substr( $self->{bufferA}, 0, $rsize, '' );
+        substr( $self->{bufferB}, 0, $rsize, '' ) if $self->{mode_B};
+        last;
+      }
     }
   }
   return 1;
@@ -207,9 +214,11 @@ _fill_buffer => <<'ENDC',
     my $buffer;
     my $result;
     # Getting new portion of data
-    $result = $self->_read( \$buffer,
-      ( $self->{buffsize} > $self->{inlimit} )? $self->{inlimit} : $self->{buffsize}
-    );
+    if( $self->{buffsize} > $self->{inlimit} ) {
+      $result = $self->_read( \$buffer, $self->{inlimit} );
+    } else {
+      $result = $self->_read( \$buffer, $self->{buffsize} );
+    }
     # Checking data
     if( !defined($result) or ($] >= 5.008001
       and !$self->{mode_U} and $result and utf8::is_utf8($buffer) and !utf8::valid($buffer)
@@ -237,15 +246,14 @@ _read => <<'ENDC',
 
   # Checking type of stream:
   #  - if SCALAR, then copy part of data from SCALAR variable
-  #  - if TYPEGLOB, then reading next part of data from file stream
   if( UNIVERSAL::isa($self->{input},'SCALAR') ) {
     $result = length(${$self->{input}}) - $self->{inpos};
     $result = $length if $result > $length;
     $result = 0 if $result < 0;
     $$strref = substr( ${$self->{input}}, $self->{inpos}, $result );
     $self->{inpos} += $result;
-  }
-  elsif( UNIVERSAL::isa($self->{input},'GLOB') ) {
+  #  - if TYPEGLOB, then reading next part of data from file stream
+  } elsif( UNIVERSAL::isa($self->{input},'GLOB') ) {
     $result = read( $self->{input}, $$strref, $length );
   }
   return $result;
@@ -265,8 +273,7 @@ _write => <<'ENDC',
   if( UNIVERSAL::isa($output,'SCALAR') ) {
     $$output .= $$strref;
     $result = 1; # alltimes true result
-  }
-  elsif ( UNIVERSAL::isa($output,'GLOB') ) {
+  } elsif ( UNIVERSAL::isa($output,'GLOB') ) {
     $result = print( {$output} $$strref );
   }
   return $result;
@@ -274,21 +281,44 @@ ENDC
 
 };
 
+# Compiling all under mod_perl
+if( exists $ENV{MOD_PERL} ) {
+  _compile($_) foreach( keys %{$CODE} );
+}
+
+# Standard function
 sub AUTOLOAD {
-  my($name) = $AUTOLOAD =~ /([^:]+)$/;
-  unless( exists $CODE->{$name} ) {
-    _croak("Undefined subroutine &$AUTOLOAD called") if $^W;
+  my $name = substr(
+    $AUTOLOAD, rindex( $AUTOLOAD, ':' ) + 1
+  );
+  unless( _compile($name) ) {
+    _croak("Undefined subroutine &${AUTOLOAD} called");
   } else {
-    eval "sub $name { ".delete($CODE->{$name})." }";
-    if( $@ and $^W ) {
-      warn $@; # it is better then nothing..
-    }
-    goto &$AUTOLOAD;
+    goto &{$AUTOLOAD};
   }
 }
 
-# Handling warnings
-sub _croak { require Carp; Carp::croak(shift) }
+# Private function: BOOL = _compile(NAME)
+# Compiling, specified by NAME, subroutine from $CODE array
+sub _compile {
+  my $name = shift;
+
+  unless( exists $CODE->{$name} ) {
+    return undef;
+  } else {
+    eval "sub $name { $CODE->{$name} }";
+    if( $@ ne '' and $^W ) {
+      warn $@; # warnings enable
+    }
+    delete $CODE->{$name};
+    return 1;
+  }
+}
+
+# Handling fatals
+sub _croak {
+  require Carp; Carp::croak(shift);
+}
 
 1;
 
@@ -348,79 +378,77 @@ for "on the fly" parsing big volumes data.
 
 =over 4
 
-=item OBJ = Stream::Reader->new( INPUT, { ... Params ... } )
+=item B<OBJ = Stream::Reader-E<gt>new( INPUT, { ... Params ... } )>
 
 The constructor method instantiates a new Stream::Reader object.
 
-INPUT - is a reference to file stream, opened for reading,
+B<INPUT> - is a reference to file stream, opened for reading,
 or reference to defined string. This is an obligatory parameter.
 
-Params (all optionaly):
+B<Params (all optionaly):>
 
 =over 2
 
-Limit - limit size of input stream data in characters. If this
+B<Limit> - limit size of input stream data in characters. If this
 parameter is absent, not defined or less then zero, then all data
 from input stream will be available for reading.
 
-BuffSize - size of buffer in characters. If this parameter is absent,
+B<BuffSize> - size of buffer in characters. If this parameter is absent,
 not defined or less then zero, then will be used default buffer size
 32768 characters.
 
-FLAGS - 2 modificators are available in this method:
-
-Mode - is string with letters-modificators:
+B<Mode> - is string with letters-modificators:
 
 =over 2
 
-B - use second buffer. Can really speed up search in case
+B<B> - use second buffer. Can really speed up search in case
 insensitive mode.
 
-U - disable UTF-8 data check in UTF-8 mode. Use this flag if you are
+B<U> - disable UTF-8 data check in UTF-8 mode. Use this flag if you are
 absolutely sure, that your UTF-8 data is valid.
 
 =back
 
 =back
 
-=item RESULT = OBJ->readto( DELIMITER, { ... Params ... } )
+=item B<RESULT = OBJ-E<gt>readto( DELIMITER, { ... Params ... } )>
 
 This method reads all data from input stream before first found
 delimiter, beginning from current position.
 
-RESULT - boolean value. True value if successfuly found delimeter
+B<RESULT> - boolean value. True value if successfuly found delimeter
 or and of input stream has expected at first time. False value
 otherwise, or in case of reading error.
 
-DELIMETER - is a string-delimeter or reference to array with
+B<DELIMETER> - is a string-delimeter or reference to array with
 many delimeters. This is an obligatory parameter and must be
 defined.
 
 Remember! In case of many delimiters, left delimiter alltimes have
 more priority then right!
 
-Params (all optionaly):
+B<Params (all optionaly):>
 
 =over 2
 
-Out - is a reference to file stream, opened for writing,
+B<Out> - is a reference to file stream, opened for writing,
 or reference to string. If this parameter is absent then data
 will not stored.
 
-Limit - size in characters. Defines, the maximum number of
+B<Limit> - size in characters. Defines, the maximum number of
 characters that must be stored in Out. If this paramter is absent,
 not defined or less then zero, then this method will be trying to
 store all readed data.
 
-Mode - is string with letters-modificators:
+B<Mode> - is string with letters-modificators:
 
 =over 2
 
-A - appendig data to Out if Out is a reference to string.
+B<A> - appendig data to Out if Out is a reference to string.
 
-I - search in case insensitive mode.
+B<I> - search in case insensitive mode.
 
-E - at the end of input stream returns only false value. Without this
+B<E> - at the end of input stream returns only false value. Without this
 modificator, if end of stream expected at first time, then will be
 returned true value.
 
@@ -428,49 +456,49 @@ returned true value.
 
 =back
 
-=item RESULT = OBJ->readsome( LIMIT, { ... Params ... } )
+=item B<RESULT = OBJ-E<gt>readsome( LIMIT, { ... Params ... } )>
 
 This method reads fixed number of characters from input stream
 beginning from current position.
 
-RESULT - boolean value. True value, if any characters were read or
+B<RESULT> - boolean value. True value, if any characters were read or
 end of input stream is not expected yet. False value otherwise, or
 in case of reading error.
 
-LIMIT - limit size in characters, how many it is necessary to read.
+B<LIMIT> - limit size in characters, how many it is necessary to read.
 If this parameter is absent, not defined or less then zero, then will
 be read all available data from input stream.
 
-Params (all optionaly):
+B<Params (all optionaly):>
 
 =over 2
 
-Out - the same as in readto() method.
+B<Out> - the same as in readto() method.
 
-Mode - is string with letters-modificators:
+B<Mode> - is string with letters-modificators:
 
 =over 2
 
-A - the same as in readto() method.
+B<A> - the same as in readto() method.
 
 =back
 
 =back
 
-=item Statistics:
+=item B<Statistics>:
 
-OBJ->{Total} - total number of readed characters. Warning! This
+B<OBJ-E<gt>{Total}> - total number of readed characters. Warning! This
 module using block reading and real position in stream is different.
 
-OBJ->{Readed} - number of readed characters at last operation
+B<OBJ-E<gt>{Readed}> - number of readed characters at last operation
 (without matched string length at readto() method).
 
-OBJ->{Stored} - number of succesfully stored chars at last operation
+B<OBJ-E<gt>{Stored}> - number of succesfully stored chars at last operation
 
-OBJ->{Match} - matched string at last operation (actually for
+B<OBJ-E<gt>{Match}> - matched string at last operation (actually for
 readto() only)
 
-OBJ->{Error} - boolen error status. At any reading erorrs
+B<OBJ-E<gt>{Error}> - boolen error status. At any reading erorrs
 all operations will be stopes and this flag turned to true value.
 
 =back
@@ -498,41 +526,30 @@ characters, not bytes!
 
 =head1 EXAMPLES
 
-=head2 Reading configuration file:
+=over 4
 
-  open( my $fh, '<', 'config.txt' ) or die $!;
-  my $stream = Stream::Reader->new($fh);
+=item *
 
-  my $line;
-  while( $stream->readto( "\r\n", { Out => \$line } ) ) {
-    # Do something with $line
-  }
-  close($fh) or die $!;
+Please visit catalogue examples/ in this module distribution.
 
-=head2 Find first one of substrings in file without case sensitive:
+=item *
 
-  # Initialize array of strings
-  my @strings = ( 'word1', 'word2', 'phrase 1', 'word3' );
+Good example is module PCGI
 
-  open( my $fh, '<', 'file.txt' ) or die $!;
-  my $stream = Stream::Reader->new($fh);
+=back
 
-  # Now, let trying to find one of substrings
-  my $r = $stream->readto( \@strings, { Mode => 'IE' } );
+=head1 CREDITS
 
-  if( $r ) {
-    print "Found substring '$stream->{Match}'\n";
-  } elsif( $stream->{Error} ) {
-    print "Fatal error during reading file!\n";
-  } else {
-    print "Nothing found..\n";
-  }
-  close($fh) or die $!;
+Special thanks to:
 
-=head1 Special thanks too:
+=over 4
+
+=item *
 
 Andrey Fimushkin, E<lt>plohaja@mail.ruE<gt>
 
+=back
+
 =head1 AUTHOR
 
 Andrian Zubko aka Ondr, E<lt>ondr@cpan.orgE<gt>
